@@ -1,5 +1,5 @@
 /**
- * Copyright (C) 2013-2019  Charlie Sharpsteen, Stefan Löffler
+ * Copyright (C) 2013-2022  Charlie Sharpsteen, Stefan Löffler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -14,10 +14,12 @@
 #ifndef PDFDocumentView_H
 #define PDFDocumentView_H
 
-#include <QtWidgets>
+#include "PDFBackend.h"
+#include "PDFDocumentTools.h"
+#include "PDFRuler.h"
 
-#include <PDFBackend.h>
-#include <PDFDocumentTools.h>
+#include <QtWidgets>
+#include <memory>
 
 namespace QtPDF {
 
@@ -47,6 +49,7 @@ class PDFDocumentView : public QGraphicsView {
   int _currentSearchResult{-1};
   QBrush _searchResultHighlightBrush;
   QBrush _currentSearchResultHighlightBrush;
+  PDFRuler _ruler{this};
   bool _useGrayScale{false};
 
   friend class DocumentTool::AbstractTool;
@@ -74,7 +77,7 @@ public:
   // They are fully wired to this PDFDocumentView (e.g., clicking on entries in
   // the table of contents will change this view)
   QDockWidget * dockWidget(const Dock type, QWidget * parent = nullptr);
-  
+
   DocumentTool::AbstractTool * armedTool() const { return _armedTool; }
   void triggerContextClick(const int page, const QPointF pos) { emit contextClick(page, pos); }
 
@@ -96,6 +99,9 @@ public:
 
   bool canGoPrevViewRects() const { return !_oldViewRects.empty(); }
 
+  bool isRulerVisible() const { return _ruler.isVisibleTo(this); }
+  PDFRuler * ruler() { return &_ruler; }
+
 public slots:
   void goPrev();
   void goNext();
@@ -145,6 +151,8 @@ public slots:
   void armTool(const DocumentTool::AbstractTool::Type toolType);
   void disarmTool();
 
+  void showRuler(const bool show = true);
+
 signals:
   void changedPage(int pageNum);
   void changedZoom(qreal zoomLevel);
@@ -153,6 +161,8 @@ signals:
   // has changed (e.g., if it was unlocked)
   void changedDocument(const QWeakPointer<QtPDF::Backend::Document> newDoc);
 
+  void updated();
+
   void searchProgressChanged(int percent, int occurrences);
   void searchResultHighlighted(const int pageNum, const QList<QPolygonF> region);
   void textSelectionChanged(const bool isTextSelected);
@@ -172,10 +182,11 @@ protected:
   void mouseReleaseEvent(QMouseEvent * event) override;
   void wheelEvent(QWheelEvent * event) override;
   void changeEvent(QEvent * event) override;
-  
+  void resizeEvent(QResizeEvent * event) override;
+
   // Maybe this will become public later on
   // Ownership of tool is transferred to PDFDocumentView
-  void registerTool(DocumentTool::AbstractTool * tool);
+  void registerTool(std::unique_ptr<DocumentTool::AbstractTool> tool);
 
   DocumentTool::AbstractTool * getToolByType(const DocumentTool::AbstractTool::Type type);
 
@@ -192,7 +203,6 @@ protected slots:
   void goToPage(const PDFPageGraphicsItem * page, const QPointF anchor, const int alignment = Qt::AlignHCenter | Qt::AlignVCenter);
   void searchResultReady(int index);
   void searchProgressValueChanged(int progressValue);
-  void switchInterfaceLocale(const QLocale & newLocale);
   void reinitializeFromScene();
   void notifyTextSelectionChanged();
 
@@ -200,14 +210,13 @@ private:
   PageMode _pageMode{PageMode_OneColumnContinuous};
   MouseMode _mouseMode{MouseMode_Move};
   QCursor _hiddenCursor;
-  QVector<DocumentTool::AbstractTool*> _tools;
+  // Use std::vector instead of QVector as the latter can't handle non-copyable
+  // types
+  std::vector< std::unique_ptr<DocumentTool::AbstractTool> > _tools;
   DocumentTool::AbstractTool * _armedTool{nullptr};
-  QMap<uint, DocumentTool::AbstractTool*> _toolAccessors;
+  QMap<uint, DocumentTool::AbstractTool::Type> _toolAccessors;
 
   QStack<PDFDestination> _oldViewRects;
-  
-  static QTranslator * _translator;
-  static QString _translatorLanguage;
 
   // Never try to set a vanilla QGraphicsScene, always use a PDFGraphicsScene.
   void setScene(QGraphicsScene *scene);
@@ -245,7 +254,7 @@ public:
 protected:
   void wheelEvent(QWheelEvent * event) override { event->ignore(); }
   void paintEvent(QPaintEvent * event) override;
-  
+
   QPixmap _dropShadow;
 };
 
@@ -261,8 +270,10 @@ public:
   // put into a QScrollArea instead).
 public slots:
   void setWindowTitle(const QString & windowTitle);
+#if QT_VERSION < QT_VERSION_CHECK(5, 2, 0)
 signals:
   void windowTitleChanged(const QString &);
+#endif
 protected slots:
   virtual void initFromDocument(const QWeakPointer<QtPDF::Backend::Document> doc) { _doc = doc; }
   virtual void retranslateUi() { }
@@ -301,7 +312,7 @@ class PDFMetaDataInfoWidget : public PDFDocumentInfoWidget
 public:
   PDFMetaDataInfoWidget(QWidget * parent);
   ~PDFMetaDataInfoWidget() override = default;
-  
+
 protected slots:
   void initFromDocument(const QWeakPointer<QtPDF::Backend::Document> doc) override;
   void clear() override;
@@ -330,7 +341,7 @@ class PDFFontsInfoWidget : public PDFDocumentInfoWidget
 public:
   PDFFontsInfoWidget(QWidget * parent);
   ~PDFFontsInfoWidget() override = default;
-  
+
 protected slots:
   void initFromDocument(const QWeakPointer<QtPDF::Backend::Document> doc) override;
   void clear() final;
@@ -351,7 +362,7 @@ class PDFPermissionsInfoWidget : public PDFDocumentInfoWidget
 public:
   PDFPermissionsInfoWidget(QWidget * parent);
   ~PDFPermissionsInfoWidget() override = default;
-  
+
 protected slots:
   void initFromDocument(const QWeakPointer<QtPDF::Backend::Document> doc) override;
   void clear() final;
@@ -377,7 +388,7 @@ class PDFAnnotationsInfoWidget : public PDFDocumentInfoWidget
 public:
   PDFAnnotationsInfoWidget(QWidget * parent);
   ~PDFAnnotationsInfoWidget() override = default;
-    
+
 protected slots:
   void initFromDocument(const QWeakPointer<QtPDF::Backend::Document> newDoc) override;
   void clear() override;
@@ -461,8 +472,8 @@ public:
   QWeakPointer<Backend::Document> document();
   QList<QGraphicsItem*> pages();
   QList<QGraphicsItem*> pages(const QPolygonF &polygon);
-  QGraphicsItem* pageAt(const int idx);
-  QGraphicsItem* pageAt(const QPointF &pt);
+  QGraphicsItem* pageAt(const int idx) const;
+  QGraphicsItem* pageAt(const QPointF &pt) const;
   int pageNumAt(const QPolygonF &polygon);
   int pageNumAt(const QPointF &pt);
   int pageNumFor(const PDFPageGraphicsItem * const graphicsItem) const;
