@@ -83,7 +83,6 @@ using vm::array;
 
 void runFile(const string& filename);
 
-
 namespace settings {
 
 using camp::pair;
@@ -993,6 +992,7 @@ struct versionOption : public option {
     bool ssbo=false;
     bool gsl=false;
     bool fftw3=false;
+    bool eigen=false;
     bool xdr=false;
     bool curl=false;
     bool lsp=false;
@@ -1022,6 +1022,10 @@ struct versionOption : public option {
     fftw3=true;
 #endif
 
+#ifdef HAVE_EIGEN_DENSE
+    eigen=true;
+#endif
+
 #ifdef HAVE_RPC_RPC_H
     xdr=true;
 #endif
@@ -1066,6 +1070,7 @@ struct versionOption : public option {
     feature("SSBO     GLSL shader storage buffer objects",ssbo);
     feature("GSL      GNU Scientific Library (special functions)",gsl);
     feature("FFTW3    Fast Fourier transforms",fftw3);
+    feature("Eigen    Eigenvalue library",eigen);
     feature("XDR      External Data Representation (portable binary file format for V3D)",xdr);
     feature("CURL     URL support",curl);
     feature("LSP      Language Server Protocol",lsp);
@@ -1291,6 +1296,9 @@ void initSettings() {
                             "Compute indexing partial sums on GPU", true));
   addOption(new boolSetting("GPUinterlock", 0,
                             "Use fragment shader interlock", true));
+  addOption(new boolSetting("GPUcompress", 0,
+                            "Compress GPU transparent fragment counts",
+                            false));
   addOption(new IntSetting("GPUlocalSize", 0, "n",
                            "Compute shader local size", 16));
 
