@@ -72,7 +72,7 @@ typedef enum { FE_FAIL, FE_RETURN_NULL } file_error_mode;
 #  include "lua.h"
 #  include "lauxlib.h"
 
-# include "luapplib/src/pplib.h"
+# include "pplib.h"
 
 
 /* pdfgen.w */
