@@ -523,13 +523,6 @@ void init_default_kanji (const_string file_str, const_string internal_str)
         if (!set_enc_string (p, NULL))
             fprintf (stderr, "Ignoring bad kanji encoding \"%s\".\n", p);
     }
-
-    p = kpse_var_value ("guess_input_kanji_encoding");
-    if (p) {
-        if (*p == '1' || *p == 'y' || *p == 't')
-            infile_enc_auto = 1;
-        free(p);
-    }
 }
 
 void init_default_kanji_select(void)
